package model.group;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import model.fixture.DmxValues;
import model.fixture.Fixture;
import model.fixture.FixtureRepository;
import model.generic.PropertyItem;
import model.lightController.LightControllable;
import model.lightingScene.LightingScene;
import model.lightingScene.LightingScene.LightingSceneType;
import model.lightingScene.composition.Composition;
import model.lightingScene.composition.CompositionRepository;
import model.lightingScene.lightingMood.LightingMood;
import model.lightingScene.lightingMood.LightingMoodRepository;

public class Group extends PropertyItem{
	private byte id;
	private String name;
	private State currentState = State.DEAKTIV;
    private LightingScene currentLightingScene;
    private LightControllable lightController;
    private FixtureRepository fixtureRepository;
    private LightingMoodRepository lightingMoodRepository;
    private CompositionRepository compositionRepository;
    private PropertyChangeListener fixtureRepositoryListener;

    public static enum State {
		AKTIV,
		PARTAKTIV,
		DEAKTIV
	}
    
	
	public Group(String name, LightControllable lightController) {
		super();
		this.name = name;
		this.lightController = lightController;
        this.currentLightingScene = null;
        this.fixtureRepository = new FixtureRepository();
        this.lightingMoodRepository = new LightingMoodRepository();
        this.compositionRepository = new CompositionRepository();
        this.fixtureRepositoryListener = new PropertyChangeListener() {
        	@Override
            public void propertyChange(PropertyChangeEvent event) {
        		if(event.getPropertyName().equals("activeFixtures")) {
    				int activeFixtures= (int)event.getNewValue();
    				if(activeFixtures == 0) {
                        setCurrentState(State.DEAKTIV);
                    } else if(activeFixtures < fixtureRepository.getFixtureMap().size()) {
                        setCurrentState(State.PARTAKTIV);
                    } else if(activeFixtures == fixtureRepository.getFixtureMap().size()) {
                        setCurrentState(State.AKTIV);
                    }
    			}
            }
        };
	}

	public byte getId() {
		return this.id;
	}

    public void setId(byte id) {
        this.propertySupport.firePropertyChange("id", this.id, id);
        this.id = id;
	}

	public String getName() {
		return this.name;
	}

    public void setName(String name) {
        this.propertySupport.firePropertyChange("name", this.name, name);
		this.name = name;
	}
	
	public State getCurrentState() {
		return this.currentState;
	}

	public void setCurrentState(State currentState) {
		this.propertySupport.firePropertyChange("currentState", this.currentState, currentState);
        this.currentState = currentState;
	}

	public void addFixture(Fixture fixture) {
		this.fixtureRepository.addFixture(fixture);
        this.compositionRepository.getCompositionMap().forEach((String name, Composition composition) -> {
            composition.setDmxValuesForFixture(fixture.getName(), new DmxValues());
        });
		
        if(this.currentLightingScene != null) {
        	if(this.currentLightingScene.getLightingSceneType() == LightingSceneType.LIGHTING_MOOD) {
	            fixture.setDmxValues(((LightingMood) this.currentLightingScene).getDmxValues());
	        } else if(this.currentLightingScene.getLightingSceneType() == LightingSceneType.COMPOSITION) {
	            fixture.setDmxValues(((Composition) this.currentLightingScene).getDmxValuesForFixture(fixture.getName()));
	        }
		}
		this.fixtureRepository.addPropertyChangeListener(this.fixtureRepositoryListener);
        if(this.currentState == State.AKTIV) {
			fixture.activate();
		}
	}
	
	public void addLightingMood(LightingMood lightingMood) {
		this.lightingMoodRepository.addLightingMood(lightingMood);
	}
	
	public void addComposition(Composition composition) {
		this.compositionRepository.addComposition(composition);
	}
	
	public void removeFixture(Fixture fixture) {
		fixture.removePropertyChangeListener(this.fixtureRepositoryListener);
		this.fixtureRepository.removeFixture(fixture.getName());
		this.compositionRepository.getCompositionMap().forEach((String name, Composition composition) -> {
            composition.removeFixtureFromComposition(fixture.getName());
        });
        fixture.deactivate();
	}
	
	public void removeLightingMood(LightingMood lightingMood) {
		this.lightingMoodRepository.removeLightingMood(lightingMood.getName());
	}
	
	public void removeComposition(Composition composition) {
		this.compositionRepository.removeComposition(composition.getName());
	}
	
	public void activateLightingMood(LightingMood lightingMood) {
        if(lightingMood != null) {
        	if(this.currentLightingScene != null) {
        		this.currentLightingScene.deactivate();
	        }
        	this.currentLightingScene = lightingMood;
	        if(!lightingMood.isActive()) {
	        	lightingMood.activate();
	        }
        	
	
	        this.fixtureRepository.getFixtureMap().forEach((String name, Fixture fixture) -> {
	            fixture.setDmxValues(lightingMood.getDmxValues());
	        });
        }
		if(this.currentState != State.DEAKTIV) {
			this.lightController.sendDmxValue();	
		}
	}
	
	public void deactivateLightingMood(LightingMood lightingMood) {
		lightingMood.deactivate();
		this.fixtureRepository.getFixtureMap().forEach((String name, Fixture fixture) -> {
            fixture.setDmxValues(new DmxValues());
        });
		this.lightController.sendDmxValue();
	}
	
	public void activateComposition(Composition composition) {
        if(this.currentLightingScene != null) {
        	this.currentLightingScene.deactivate();
        }

        this.currentLightingScene = composition;		
		if(!composition.isActive()) {
			composition.activate();
		}
       

		this.fixtureRepository.getFixtureMap().forEach((String name, Fixture fixture) -> {
            fixture.setDmxValues(composition.getDmxValuesForFixture(name));
        });
		if(this.currentState != State.DEAKTIV) {
			this.lightController.sendDmxValue();	
		}
	}
	
	public void deactivateComposition(Composition composition) {
		composition.deactivate();
		this.fixtureRepository.getFixtureMap().forEach((String name, Fixture fixture) -> {
            fixture.setDmxValues(new DmxValues());
        });
		this.lightController.sendDmxValue();
	}
	
	public FixtureRepository getFixtureRepository() {
		return this.fixtureRepository;
	}
	
    public LightingMoodRepository getLightingMoodRepository() {
        return this.lightingMoodRepository;
    }

    public CompositionRepository getCompositionRepository() {
        return this.compositionRepository;
    }
	
	public void activate() {
		this.fixtureRepository.getFixtureMap().forEach((String name, Fixture fixture) -> {
            fixture.activate();
        });
		this.lightController.sendDmxValue();
		this.lightController.activateGroup(this.id);
	}
	
	public void deactivate() {
		this.fixtureRepository.getFixtureMap().forEach((String name, Fixture fixture) -> {
            fixture.deactivate();
        });
		this.lightController.sendDmxValue();
		this.lightController.deactivateGroup(this.id);
	}
}