package model.lightController.artNet.udpListener;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.nio.ByteBuffer;
import java.util.concurrent.atomic.AtomicBoolean;
import com.google.gson.Gson;

import model.lightController.artNet.ArtNetController;
import model.lightController.artNet.artNetCommands.PresetActivatedCommand;
import model.lightController.artNet.artNetCommands.PresetDeactivatedCommand;
import model.lightController.artNet.artNetNode.ArtNetNode;
import model.lightController.artNet.artNetPakages.ArtCommand;
import model.lightController.artNet.artNetPakages.ArtDataReply;
import model.lightController.artNet.artNetPakages.ArtNetCodeLibrary;
import model.lightController.artNet.artNetPakages.ArtPollReply;
import model.preset.PresetRepository;

public class UdpListener implements Runnable{
	
	private ArtNetController artNetController;
	private PresetRepository presetRepository;
	private int port = 6454;
	private byte[] opCodeArray = new byte[2];
	private short opCode;
	byte[] incomingData;
	AtomicBoolean running;
	Gson gson;
	
	public UdpListener(ArtNetController artNetController, int port, PresetRepository presetRepository, AtomicBoolean running) {
		this.artNetController = artNetController;
		this.presetRepository = presetRepository;
		this.running = running;
		incomingData = new byte[550];
		this.gson = new Gson();
	}
	
	private boolean checkIfArtNetPakage(byte[] incomingData) {
		boolean isArtNet = true;
		byte[] artNetString = ("Art-Net" + (char)0x00).getBytes();
		
		for(int i = 0; i<8; i++) {
			if(artNetString[i] != incomingData[i]) {
				isArtNet = false;
			}
		}
		return isArtNet;
	}
	
	private String convertIp(byte[] ip) {
		String ipString = "";
		for(int i = 0; i<3; i++) {
			ipString += Byte.toUnsignedInt(ip[i]) + ".";
		}
		ipString += Byte.toUnsignedInt(ip[3]);
		return ipString;
	}
	
	private void sendArtPollReply(InetAddress reciverIP) {
		byte[] udpSendData = (new ArtPollReply()).getUdpArray();
        
        try(DatagramSocket socket = new DatagramSocket()) {
            DatagramPacket outgoingDatagram = new DatagramPacket(udpSendData, udpSendData.length, reciverIP, this.port);
            socket.send(outgoingDatagram);
        } catch (SocketException e) {
            e.printStackTrace(System.err);
        } catch (IOException e) {
            e.printStackTrace(System.err);
        }
	}
	
	private void processArtPollReply() {
		System.out.println("ArtPollReply empfangen");
		ArtPollReply artPollReply = new ArtPollReply(incomingData);
		artNetController.getNodeRepository().getNodesMap().forEach((String name, ArtNetNode node) -> {
			if(convertIp(artPollReply.getIpAddress()).equals(node.getIp())) {
				node.setConnectStatus(true);
			}
		});
	}
	
	private void processArtCommand() {
		ArtCommand artCommand = new ArtCommand(incomingData);
		String commandsString = artCommand.getData();
		String[] commands = commandsString.split("&");
		for(String commandString : commands) {
			String command = commandString.split("=")[0];
			String commandJSON = commandString.split("=")[1].trim();
			
			if(command.equals(ArtNetCodeLibrary.ArtCommands.ART_COM_PA.command)) {
				PresetActivatedCommand presetActivatedCommand = this.gson.fromJson(commandJSON, PresetActivatedCommand.class);
				presetRepository.getPreset(presetActivatedCommand.getPresetId()).activate();
				
			} else if(command.equals(ArtNetCodeLibrary.ArtCommands.ART_COM_PD.command)) {
				PresetDeactivatedCommand presetDeactivatedCommand = this.gson.fromJson(commandJSON, PresetDeactivatedCommand.class);
				presetRepository.getPreset(presetDeactivatedCommand.getPresetId()).deactivate();	
			}	
		}	
	}
	
	private void processArtDataReply() {
		ArtDataReply artDataReply = new ArtDataReply(incomingData);
		if(artDataReply.getRequestCode() == ArtNetCodeLibrary.RequestCodes.DR_PRESET_STATUS.requestCode) {
			for(int i = 0; i < (int)artDataReply.getPayLength(); i++) {
				if(artDataReply.getPayLoad()[i] == 1) {
					presetRepository.getPreset(i).activate();
				}
			}
		}
	}
	
	@Override
	public void run() {
		
		try(DatagramSocket socket = new DatagramSocket(this.port)) {
			socket.setSoTimeout(1000);
			while(this.running.get()) {
				try {
					DatagramPacket incomingDatagram = new DatagramPacket(this.incomingData, this.incomingData.length);
					socket.receive(incomingDatagram);
					this.incomingData = incomingDatagram.getData();
					
					if(checkIfArtNetPakage(this.incomingData)) {
						opCodeArray[0] = this.incomingData[9];
						opCodeArray[1] = this.incomingData[8];
						this.opCode = ByteBuffer.wrap(opCodeArray).getShort();
						if(this.opCode == ArtNetCodeLibrary.OpCodes.OP_POLL.opCode) {
							//System.out.println("Art Poll empfangen.");
							this.sendArtPollReply(incomingDatagram.getAddress());
						}else if(this.opCode == ArtNetCodeLibrary.OpCodes.OP_POLL_REPLY.opCode) {
							//System.out.println("Art Poll Reply empfangen.");
							this.processArtPollReply();
						} else if(this.opCode == ArtNetCodeLibrary.OpCodes.OP_COMMAND.opCode) {
							System.out.println("Art Command empfangen.");
							this.processArtCommand();
						} else if(this.opCode == ArtNetCodeLibrary.OpCodes.OP_DATA_REPLY.opCode) {
							//System.out.println("Art Data Reply empfangen.");
							this.processArtDataReply();
						}
					}
				} catch (SocketTimeoutException e) {}
			}
			socket.close();
		} catch (SocketException e) {
			e.printStackTrace(System.err);
		} catch (IOException e) {
			e.printStackTrace(System.err);
		} finally {
			artNetController.getNodeRepository().getNodesMap().forEach((String name, ArtNetNode node) -> {
				node.setConnectStatus(false);
			});
		}
	}

}
